/**
 * Copyright 2007 - 2010 Skyway Software, Inc.
 * 
 * Skyway Software, Inc. licenses this file to You under the Apache License, 
 * Version 2.0(the "License"); you may not use this file except in compliance 
 * with the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.skyway.spring.util.dao;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.Query;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.transaction.annotation.Transactional;

/**
 * @see org.skyway.spring.util.dao.JpaDao
 * 
 * @author JRodriguez
 *
 */
public abstract class AbstractJpaDao implements JpaDao {
	private int defaultMaxResults = DEFAULT_MAX_RESULTS;
	private static transient Log log = LogFactory.getLog(AbstractJpaDao.class);


	/**
	 * The {@link EntityManager} which is used by all query manipulation and execution in this DAO.
	 * 
	 * @return the {@link EntityManager}
	 */
	public abstract EntityManager getEntityManager();

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#getTypes()
	 */
	public abstract Set<Class<?>> getTypes();

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#store(java.lang.Object)
	 */
	@Transactional
	public <T extends Object> T store(T toStore) {
		return getEntityManager().merge(toStore);
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#remove(java.lang.Object)
	 */
	@Transactional
	public void remove(Object toRemove) {
		toRemove = getEntityManager().merge(toRemove);
		getEntityManager().remove(toRemove);
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#flush()
	 */
	@Transactional
	public void flush() {
		getEntityManager().flush();
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#refresh(java.lang.Object)
	 */
	@SuppressWarnings("unchecked")
	@Transactional
	public void refresh(Object o) {
		try {
			if (o != null) {
				if (o instanceof java.util.Collection) {
					for (Iterator<?> i = ((Collection<?>) o).iterator(); i.hasNext();) {
						try {
							refresh(i.next());
						} catch (EntityNotFoundException x) {
							// This entity has been deleted - remove it from the collection
							i.remove();
						}
					}
				} else {
					if (getTypes().contains(o.getClass())) {
						getEntityManager().refresh(o);
					}
				}
			}
		} catch (EntityNotFoundException x) {
			// This entity has been deleted
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#setDefaultMaxResults(int)
	 */
	@Transactional
	public void setDefaultMaxResults(int defaultMaxResults) {
		this.defaultMaxResults = defaultMaxResults;
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#getDefaultMaxResults()
	 */
	@Transactional
	public int getDefaultMaxResults() {
		return defaultMaxResults;
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#executeQueryByNameSingleResult(java.lang.String)
	 */
	@Transactional
	@SuppressWarnings("unchecked")
	public <T extends Object> T executeQueryByNameSingleResult(String queryName) {
		return (T) executeQueryByNameSingleResult(queryName, (Object[])null);
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#executeQueryByNameSingleResult(java.lang.String, java.lang.Object[])
	 */
	@Transactional
	@SuppressWarnings("unchecked")
	public <T extends Object> T executeQueryByNameSingleResult(String queryName, Object... parameters) {
		Query query = createNamedQuery(queryName, DEFAULT_FIRST_RESULT_INDEX, 1, parameters);
		return (T) query.getSingleResult();
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#executeQueryByName(java.lang.String)
	 */
	@Transactional
	public <T extends Object> List<T> executeQueryByName(String queryName) {
		return executeQueryByName(queryName, DEFAULT_FIRST_RESULT_INDEX, getDefaultMaxResults());
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#executeQueryByName(java.lang.String, java.lang.Integer, java.lang.Integer)
	 */
	@Transactional
	public <T extends Object> List<T> executeQueryByName(String queryName, Integer firstResult, Integer maxResults)  {
		return executeQueryByName(queryName, firstResult, maxResults, (Object[])null);
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#executeQueryByName(java.lang.String, java.lang.Object[])
	 */
	@Transactional
	public <T extends Object> List<T> executeQueryByName(String queryName, Object... parameters)  {
		return executeQueryByName(queryName, DEFAULT_FIRST_RESULT_INDEX, getDefaultMaxResults(), parameters);
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#executeQueryByName(java.lang.String, java.lang.Integer, java.lang.Integer, java.lang.Object[])
	 */
	@Transactional
	@SuppressWarnings("unchecked")
	public <T extends Object> List<T> executeQueryByName(String queryName, Integer firstResult, Integer maxResults, Object... parameters)  {
		Query query = createNamedQuery(queryName, firstResult, maxResults, parameters);
		return query.getResultList();
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#createNamedQuery(java.lang.String, java.lang.Integer, java.lang.Integer)
	 */
	@Transactional
	public Query createNamedQuery(String queryName, Integer firstResult, Integer maxResults)  {
		return createNamedQuery(queryName, firstResult, maxResults, (Object[])null);
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#createNamedQuery(java.lang.String, java.lang.Integer, java.lang.Integer, java.lang.Object[])
	 */
	@Transactional
	public Query createNamedQuery(String queryName, Integer firstResult, Integer maxResults, Object... parameters)  {
		Query query = getEntityManager().createNamedQuery(queryName);
		if (parameters != null) {
			for (int i = 0; i < parameters.length; i++) {
				query.setParameter(i + 1, parameters[i]);
			}
		}
		
		query.setFirstResult(firstResult == null || firstResult < 0 ? DEFAULT_FIRST_RESULT_INDEX : firstResult);
		if (maxResults != null && maxResults > 0)
			query.setMaxResults(maxResults);

		return query;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#executeQuery(java.lang.String, java.lang.Integer, java.lang.Integer, java.lang.Object[])
	 */
	@Transactional
	@SuppressWarnings("unchecked")
	public <T extends Object> List<T> executeQuery(String queryString, Integer firstResult, Integer maxResults, Object... parameters)  {
		Query query = createQuery(queryString, firstResult, maxResults, parameters);
		return query.getResultList();
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#executeQuery(java.lang.String, java.lang.Object[])
	 */
	@Transactional
	@SuppressWarnings("unchecked")
	public <T extends Object> List<T> executeQuery(String queryString, Object... parameters)  {
		Query query = createQuery(queryString, DEFAULT_FIRST_RESULT_INDEX, getDefaultMaxResults(), parameters);
		return query.getResultList();
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#executeQuerySingleResult(java.lang.String)
	 */
	@Transactional
	@SuppressWarnings("unchecked")
	public <T extends Object> T executeQuerySingleResult(String queryString) {
		return (T) executeQuerySingleResult(queryString, (Object[])null);
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#executeQuerySingleResult(java.lang.String, java.lang.Object[])
	 */
	@Transactional
	@SuppressWarnings("unchecked")
	public <T extends Object> T executeQuerySingleResult(String queryString, Object... parameters) {
		Query query = createQuery(queryString, DEFAULT_FIRST_RESULT_INDEX, 1, parameters);
		return (T) query.getSingleResult();
	}

	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#createQuery(java.lang.String, java.lang.Integer, java.lang.Integer)
	 */
	@Transactional
	public Query createQuery (String queryString, Integer firstResult, Integer maxResults){
		return createQuery(queryString, firstResult, maxResults, (Object[])null);
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.skyway.spring.util.dao.JpaDao#createQuery(java.lang.String, java.lang.Integer, java.lang.Integer, java.lang.Object[])
	 */
	@Transactional
	public Query createQuery (String queryString, Integer firstResult, Integer maxResults, Object... parameters){
		Query query = getEntityManager().createQuery(queryString);
		if (parameters != null) {
			for (int i = 0; i < parameters.length; i++) {
				query.setParameter(i + 1, parameters[i]);
			}
		}

		query.setFirstResult(firstResult == null || firstResult < 0 ? DEFAULT_FIRST_RESULT_INDEX : firstResult);
		if (maxResults != null && maxResults > 0)
			query.setMaxResults(maxResults);

		return query;
	}
	
	
}
