/**
 * Copyright 2007 - 2008 Skyway Software, Inc.
 * 
 * Skyway Software, Inc. licenses this file to You under the Apache License, 
 * Version 2.0(the "License"); you may not use this file except in compliance 
 * with the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.skyway.spring.util.webservice.cxf;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.cxf.binding.soap.saaj.SAAJInInterceptor;
import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.message.Message;
import org.apache.cxf.phase.AbstractPhaseInterceptor;
import org.apache.cxf.phase.Phase;
import org.apache.cxf.ws.security.wss4j.WSS4JInInterceptor;
import org.apache.ws.security.handler.WSHandlerConstants;
import org.springframework.security.AuthenticationManager;

/**
 * Authenticates incoming web service calls.  If security is enabled, the AuthenticationManager
 * will be injected by spring as will the list of secured service end points.
 * 
 * @author jperkins
 *
 */
public class WSSecurityInInterceptor extends AbstractPhaseInterceptor<Message> {
	private AuthenticationManager authenticationManager;
	private List<String> securedServiceEndPointNames;
	
	public WSSecurityInInterceptor() {
		super(Phase.PRE_PROTOCOL);
	}

	public WSSecurityInInterceptor(String s) {
		super(Phase.PRE_PROTOCOL);
	}

	@SuppressWarnings("unchecked")
	public void handleMessage(Message message) throws Fault {
		/*
		 * Only check if we need to authenticate if we are not the requestor. If we are the
		 * requestor than we sent the request and it is now coming back in.
		 */
		if (!Boolean.TRUE.equals(message.get(Message.REQUESTOR_ROLE))) {
			if (isSecurityEnabled(message)) {
				Map props = new HashMap();
				props.put(WSHandlerConstants.ACTION, WSHandlerConstants.USERNAME_TOKEN);
				props.put(WSHandlerConstants.PW_CALLBACK_REF, new AuthenticateCredentialsHandler(getAuthenticationManager()));

				WSS4JInInterceptor wss4jInHandler = new WSS4JInInterceptor(props);

				message.getInterceptorChain().add(wss4jInHandler);
				message.getInterceptorChain().add(new SAAJInInterceptor());
			}
		}
	}

	private boolean isSecurityEnabled(Message message) {
		// If authenticationProvider hasn't been set, it means that security is not enabled
		if (getAuthenticationManager() == null) {
			return false;
		}
		
		// The path info has the form:
		// /<project name>/<service endpoint non-qualified interface name>
		// i.e.,
		// /ServerSecurityWS/IHelloWorldServiceServiceEndpoint
		String pathInfo = (String)message.get("org.apache.cxf.message.Message.PATH_INFO"); //$NON-NLS-1$
		
		String serviceEndpoint = StringUtils.substringAfterLast(pathInfo, "/"); //$NON-NLS-1$
		
		return getSecuredServiceEndPointNames().contains(serviceEndpoint);
	}
	
	public AuthenticationManager getAuthenticationManager() {
		return authenticationManager;
	}

	public void setAuthenticationManager(AuthenticationManager authenticationManager) {
		this.authenticationManager = authenticationManager;
	}

	public List<String> getSecuredServiceEndPointNames() {
		return securedServiceEndPointNames;
	}

	public void setSecuredServiceEndPointNames(List<String> securedServiceEndPointNames) {
		this.securedServiceEndPointNames = securedServiceEndPointNames;
	}
}
